;
; IDL Event Callback Procedures
; ZHT_Gui_eventcb
;
; v1.0.6  20100209 - added odd angle test
; v1.0.5  20100205 - changed shift decision making to gaussian probability comparison
;
;
;-----------------------------------------------------------------
pro MSP_ZHT_PostCreation, wWidget, _EXTRA=_VWBExtra_
	;
	version_short_string = 'v2.0.0 20161014 DPC'
	version_string = 'MSP_ZHT (Mimir Software Programs - Zero Hop Tester) '+version_short_string
	;
	;	set up and clear summary structure
	;
	main_struct = {		version_string: version_string, 	$	;for window title
						version_short_string: version_short_string,	$	;for inclusion on data files

						last_dir:				'',			$   ;last data directory accessed
						files_in:		strarr(500),		$
						n_files_in:				0L,			$
						flat_flag:				0L,			$
						use_originals_flag:		0L,			$
						;
						nf:						0L,			$ 	; number of files read automagically
						input_files:   strarr(1000),		$	; for passing to get_files
						;
						file_pointer:			0L,			$   ; pointer to current POLCAT file being processed
						n_pcats:				0L,			$	; number of files on list
						pcat_files:		strarr(1000),		$	; list of pathnames to POLCAT files
						mode:					0L,			$	; 0 = read CDS raw files directly; 1 = use PCATS to point to raws
						;
						single_step:			1L,			$	; 1=pause after each field; 0=no pause
						;
						lu_out:					0L,			$
						outdir:				'none',			$
						outfile:			'none',			$
						nbad:					0L,			$ 	; number of bad images in this field
						;
						;	wavelength for test
						;
						wave_band:				'',			$	;"H" or "K"
						;
						;	subdirectory image list
						;
						n_sub_files: 			0L,			$
						sub_files: 		strarr(500),		$
						sub_flag: 				0L,			$
						;
						Base_wid:				0L,			$
						base_orig_wid:			0L,			$
						print_wid:				0L,			$
						button_wid:		lonarr(10),			$
						draw_wid:		lonarr(10), 		$
						text_wid:		lonarr(10)			$
						;
					  }
	;
	PTR_main = PTR_NEW(main_struct,/NO_COPY)
    ;
    ;	load the pointer(s) into the pointer structure
    ;
    PTR_struct = {PTR_main: PTR_main}
    ;
    WIDGET_CONTROL, wWidget, SET_UVALUE=PTR_struct
end
;
;-----------------------------------------------------------------
pro MSP_ZHT_Realize, wWidget
	WIDGET_CONTROL, wWidget, GET_UVALUE=PTR_struct
	;
	;	load all the widget IDs
	;
	;	need to pass an event structure:
	;
	event = {top: wWidget}
	get_zht_wids, event
	;
	;	reread the pointer structure to get updated wids
	;
	WIDGET_CONTROL, wWidget, GET_UVALUE=PTR_struct
	m_ptr = PTR_struct.PTR_main
	;
	TLB_Title = (*m_ptr).version_string
    tlb_wid = WIDGET_INFO( Event.top, FIND_BY_UNAME='MSP_ZHT_Base_0')
    WIDGET_CONTROL, tlb_wid, TLB_SET_TITLE=TLB_Title
    ;
    ;	set the single-step button to ON
    ;
    WIDGET_CONTROL, (*m_ptr).button_wid[9], SET_BUTTON=1
    (*m_ptr).single_step = 1L
    ;
    ;	turn found originals base to not-sensitive
    ;
    WIDGET_CONTROL, (*m_ptr).base_orig_wid, SENSITIVE=0
    ;
    ;	set the initial data search directory
    ;
    cd, CURRENT=this_dir
    (*m_ptr).last_dir = this_dir
    ;
    ;	and set H-Pol as the default
    ;
    Select_Wave, Event, 0
	;
	zprint_text, event, "Mimir Software Package: Zero Hop Test Package - Initialized!"
end
;
;-----------------------------------------------------------------
pro Page_0_Handler, Event
	;
	buttons = [	"Button_File_Chooser", $		; CDS_RAWs being selected
				"Button_Start_Analysis", $
				"Button_Exit", $
				"Button_Do_Not_Use_Originals",$
				"Button_Use_Originals", $
				"Button_H_Pol", $
				"Button_K_Pol", $
				"Button_GPIPS_DR3", $
				"Button_POLCAT", $				; POLCATs being selected
				"Button_Next"  $
			   ]
	wid_button = Event.id
	n_buttons = N_ELEMENTS(buttons)
	button_wids = lonarr(n_buttons)
	for iw = 0, n_buttons-1 do button_wids[iw] = WIDGET_INFO(Event.top, FIND_BY_UNAME=buttons[iw])
	;
	ind = WHERE(button_wids[*] eq wid_button, count)
	if(count ne 0) then begin
		case wid_button of
			button_wids[0]: Get_Files, Event, 0
			button_wids[1]: Start_Analysis, Event
			button_wids[2]: Exit_Button_Press, Event
			;
			button_wids[3]: Select_Originals, Event, 0
			button_wids[4]: Select_Originals, Event, 1
			;
			button_wids[5]: Select_Wave, Event, 0
			button_wids[6]: Select_Wave, Event, 1
			;
			button_wids[7]: Process_GPIPS_DR3, Event
			button_wids[8]: Get_POLCAT_Info, Event
			button_wids[9]: Toggle_Single_Step, Event
			;
			else: zprint_text, Event, "Error - button push not recognized"
	    endcase
	endif else zprint_text, Event, "No button pushed."
end
;
;-----------------------------------------------------------------
pro Exit_Button_Press, Event
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	m_ptr = PTR_struct.PTR_main
	;
	;	close output file, if open
	;
	if( (*m_ptr).outfile ne 'none') then begin
		FREE_LUN, (*m_ptr).lu_out
	endif
	;
	;free the pointers
	;
	PTR_FREE, m_ptr
	;
	WIDGET_CONTROL, Event.top, /DESTROY
end
;
;------------------------------------------------------------------
pro Toggle_Single_Step, Event
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	m_ptr = PTR_struct.PTR_main
	;
	(*m_ptr).single_step = 1L - (*m_ptr).single_step
	WIDGET_CONTROL, (*m_ptr).button_wid[9], SET_BUTTON=(*m_ptr).single_step
	;
end
;
; Empty stub procedure used for autoloading.
;
pro ZHT_Gui_eventcb
end
;-----------------------------------------------------------------
pro Part_0_Handler, Event
	Page_0_Handler, Event
end
