pro get_directory_structure, dates, paths
;
;	when pointed to a top-level directory, this routine
;	builds a list of the directories there, then descends
;	each directory and looks for night subdirectories,
;	building a mapped list of nights to directory paths
;
;	DPC	20161005
;
top_dir = dialog_pickfile(Title='Select Top-Level Directory of Raw Data Runs',/dir)
;
;	go there and get a list of its contents
;
level_1_list = FILE_SEARCH(top_dir+'*',/MARK_DIRECTORY)
;
n_level_1 = N_ELEMENTS(level_1_list)
;
print, 'Found ',n_level_1,' directories in this main directory'
;
;stop
;
;	for each directory, go into it and get all the names there
;
level_2_list = strarr(n_level_1,100)
n_level_2 = intarr(n_level_1)
;
for i = 0, n_level_1-1 do begin
	;
	this_dir = level_1_list[i]
	;
	level_2 = FILE_SEARCH(this_dir+'*')
	;
	n_level_2[i] = N_ELEMENTS(level_2)
	;
	print, 'In subdirectory ',FILE_BASENAME(this_dir),' found ',n_level_2[i],' nights'
	;
;	stop
	;
	;	keep only those that when parsed have YYYYMMDD names - start with '2' and have 8 characters
	;
	ind = lonarr(n_level_2[i])
	ngood = 0L
	;
	for kst = 0, n_level_2[i]-1 do begin
		this_path = level_2[kst]
		;
		parts = STRSPLIT(this_path,PATH_SEP(),/extract)
		;
		np = N_ELEMENTS(parts)
		;
		stl2 = STRLEN(parts[np-1])
		stp2 = STRPOS(parts[np-1],'2')
		;
		if(stl2 eq 8 and stp2 eq 0) then begin
			ind[ngood] = kst
			ngood++
		endif
	endfor
	;
	if(ngood eq 0) then begin
		print, '    no proper night directories found, skipping'
		n_level_2[i] = 0
		level_2_list[i,*] = -1
	endif else begin
		;
		ind = ind[0:(ngood-1)]
		;
		print,'     of which ',ngood,' were proper night subdirectories'
		;
		level_2_list[i,0:(ngood-1)] = level_2[ind]
		n_level_2[i] = ngood
		;
	endelse
endfor
;
;	this should be the full list
;
;	next, sort by date into a list of pathnames
;
n_total = TOTAL(n_level_2)
;
dates = lonarr(n_total)
paths = strarr(n_total)
;
;	load them, then sort by date
;
ii = 0
for i = 0, n_level_1-1 do begin
	;
	if(n_level_2[i] gt 0) then begin
		for j = 0, n_level_2[i]-1 do begin
			;
			paths[ii] = level_2_list[i,j]
			;
			parts = STRSPLIT(paths[ii],PATH_SEP(),/extract)
			np = N_ELEMENTS(parts)
			;
			this_part = parts[np-1]
			;
			;	first character must be a '2' and the length must be 8 characters - if not, skip it
			;
			res = STRPOS(this_part,'2')
			stl = STRLEN(this_part)
			;
			if(res eq 0 and stl eq 8) then begin			; was a 2 and 8 ch long
				;
				this_date = 0L
				;
				reads, this_part, this_date, format='(i8)'
				;
				print, 'found data for night ',this_date
				;
		;		stop
				;
				dates[ii] = this_date
				paths[ii] += PATH_SEP()
				;
				ii++
			endif
		endfor
	endif
endfor
;
dates = dates[0:(ii-1)]
paths = paths[0:(ii-1)]
;
ind = sort(dates)
;
dates = dates[ind]
paths = paths[ind]
;
;print, 'Finished sorting night designations and paths for ',N_ELEMENTS(ind),' observing nights'
;
;save, dates, paths, dialog_pickfile(Title='Save file for Dates and Paths')
;
end