pro start_analysis, Event
	;
	; files_in, check_dither, fidelity_out, this_fold_angle, special_PA, debug
	;
	;	for a group of polarimetry images,
	;	evaluates HWP offset angle and recommends what to
	;	do for each image.
	;
	;	INPUT:
	;			files_in - string array of pathnames to group CDS files
	;			check_dither - integer flag (0 = skip dither checking; 1 = do dither checking)
	;	OUTPUT:
	;			fidelity_out - integer array of recommendations. Values
	;				are: "-1" = delete image
	;					 "0"  = leave image "as is"
	;				     "1" = change HWP angle to HWP+45 deg
	;			this_fold_ang - integer array holding the fold angle of the image (0, 45, 90, 135)
	;
	;	DPC	20091221 - initial version
	;	DPC 20091222 - changed to multiplicative scaling from additive
	;--------------------------------------------------------------
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	m_ptr = PTR_struct.PTR_main
	;
	;	set plot window wids
	;
	draw_wid_0 = (*m_ptr).draw_wid[0]
	WIDGET_CONTROL, draw_wid_0, GET_VALUE=win_no_0
	wset, win_no_0
	erase
	draw_wid_1 = (*m_ptr).draw_wid[1]
	WIDGET_CONTROL, draw_wid_1, GET_VALUE=win_no_1
	wset, win_no_1
	erase
	draw_wid_2 = (*m_ptr).draw_wid[2]
	WIDGET_CONTROL, draw_wid_2, GET_VALUE=win_no_2
	wset, win_no_2
	erase
	draw_wid_3 = (*m_ptr).draw_wid[3]
	WIDGET_CONTROL, draw_wid_3, GET_VALUE=win_no_3
	wset, win_no_3
	erase
	draw_wid_8 = (*m_ptr).draw_wid[8]
	WIDGET_CONTROL, draw_wid_8, GET_VALUE=win_no_8
	wset, win_no_8
	erase
	draw_wid_9 = (*m_ptr).draw_wid[9]
	WIDGET_CONTROL, draw_wid_9, GET_VALUE=win_no_9
	wset, win_no_9
	erase
	;
	;	and text window wids
	;
	text_wid_1 = (*m_ptr).text_wid[1]
	;
	;	get images
	;
	files_in = (*m_ptr).files_in[0:(*m_ptr).n_files_in - 1]
	check_dither = 1 - (*m_ptr).flat_flag
	;
	debug = 0
	;
	n_images = N_ELEMENTS(files_in)
	;
	;if(debug) then print, "number of images = ",n_images
	;
	pixel_value = fltarr(n_images)
	s_pixel_value = fltarr(n_images)
	reference_pixel_value = fltarr(n_images)
	s_reference_pixel_value = fltarr(n_images)
	hwp_value = fltarr(n_images)
	image_number = indgen(n_images)
	jd = dblarr(n_images)
	file_name_list = FILE_BASENAME(files_in)
	the_ind = lonarr(n_images)
	;
	;	for each image
	;
	this_u = fltarr(n_images)
	this_u[*] = -99.9
	this_q = fltarr(n_images)
	this_q[*] = -99.9
	this_p = fltarr(n_images)
	this_p[*] = -99.9
	this_pa = fltarr(n_images)
	this_pa[*] = -99.9
	this_fold_angle = intarr(n_images)
	this_fold_angle[*] = -99
	this_dither_cindex = intarr(n_images)
	this_dither_cindex[*] = 10
	;
	fidelity_out = intarr(n_images)
	;
	ix = 163				; this is a good pixel for tracing instrumental polarization
	iy = 850
	;
	WIDGET_CONTROL, /HOURGLASS
	;
	nom_hwp_angle = [0.0, 22.3, 45.2, 67.5, 89.8, 112.7, 135.0, 157.3, $
						 180.2, 202.5, 224.8, 247.7, 270.0, 292.3, 315.2, 337.5]
	;
	bad_image_score = intarr(2000)		; counts 'badness'
	;
	ii = 0L
	for i = 0, n_images-1 do begin
		bad_image_score[i] = 0
		;
		the_ind[i] = -1
		;
		image = readfits(files_in[i],header,/silent)
		;
		;	get HWP angle - look for some higher-level keywords first
		;
		hwp_a = sxpar(header, "HWPANGLE", COUNT=count)
		hwp_b = sxpar(header, "MB_HWP", COUNT=b_count)
		if(count+b_count gt 0) then begin
			if(b_count gt 0) then hwp_value[i] = hwp_b
			if(count gt 0) then hwp_value[i] = hwp_a
		endif else begin
			;
			;	first attempt failed, look for raw count
			;
			comment = sxpar(header, "COMMENT", COUNT=count)
			ind = WHERE(STRPOS(comment,"BD1") ne -1,ngood)
			if(ngood gt 0) then begin
				;
				comment1 = STRUPCASE( STRTRIM( comment[ind[0]],2 ))
				;
				s0 = STRPOS( comment1, "GOTO")
				s1 = STRPOS( comment1, '.')
				s3 = STRPOS( comment1, 'HOME')
				;
				istep = 0L
				if(s0 ge 0) then begin
					if(s1 ge 0) then begin
						if(s3 ge 0) then begin
							angle = 0.0
						endif else begin
							hwp_string = STRMID( comment1, s0+5, (s1-s0-5))
							reads, hwp_string, format='(i)', istep
							angle = float(istep) * 0.675d0
						endelse
					endif else begin
						;
						;	no dots... so, parse with spaces
						;
						len = STRLEN(comment1) - s0
						remainder = STRMID(comment1, s0, len-1)
						parts = STRSPLIT(remainder, ' ',/REGEX,/EXTRACT)
						len1 = STRLEN(parts[1])			; "GOTO" is parts[0]
						if(len1 ne 0) then begin
							nstep = STRTRIM(parts[1],2)
							reads, nstep, format='(i)', istep
							angle = float(istep) * 0.675d0
						endif else angle = -99.0
					endelse
				endif else begin
					if(s3 ge 0) then begin
						angle = 0.0
					endif else begin
						angle = -99.9
					endelse
				endelse
				;
				hwp_value[i] = angle
				if(angle eq -99.9) then begin
					fidelity_out[i] = -1
					bad_image_score[i] += 40
				endif
			endif else begin
				;
				;	look for new fits header variables
				;
				hwp_d = sxpar(header, "HWP_ANG", COUNT=d_count)
				if(d_count gt 0) then hwp_value[i] = hwp_d else begin
					hwp_value[i] = -99.9
					fidelity_out[i] = -1		; mark to delete
					bad_image_score[i] += 40
				endelse
			endelse
		endelse
		;
		if(hwp_value[i] ne -99.9) then begin
			d_hwp = abs(abs(hwp_value[i]) - nom_hwp_angle)
			d_hwp_min = MIN(d_hwp)
			ind_match = WHERE(d_hwp eq d_hwp_min)
			hwp_value[i] = nom_hwp_angle[ind_match[0]]
		endif
		;
		if(hwp_value[i] ne -99.9) then begin
			if(check_dither) then begin
				;-------------------------------------------------------
				;
				;	get julian date
				;
				test_jd = SXPAR(header, "MB_JD", COUNT=jd_count)
				if(jd_count eq 1) then begin
					jd[i] = test_jd
				endif else begin
					date_obs = FXPAR(header, "DATE-OBS")
					;
					;	parse date / time
					;
					reads, date_obs, year, month, day, hour, minute, second, format='(i4,1x,i2,1x,i2,1x,i2,1x,i2,1x,f4.2)
					;
					;	convert to JD
					;
					jd[i] = JULDAY(Month, Day, Year, Hour, Minute, Second)
					;
				endelse
				;-------------------------------------------------------
			endif else jd[i] = float(i)
			;
			;	get pixel value in zone
			;
			sub_image = image[ix-40:ix+40, iy-40:iy+40]
			ss = median_filtered_mean(sub_image)
			pixel_value[i] = 25000.00 < (ss[0] > (-1000.0))
			if(pixel_value[i] ne -1000.0 and pixel_value[i] ne 25000.0) then begin
				s_pixel_value[i] = ss[1] / sqrt(float(N_elements(sub_image))-1)
			endif else begin
				s_pixel_value[i] = 10000.0
				bad_image_score[i] += 40
			endelse
			;
			if(debug eq 1) then print, "image, hwp, pixel mean = ",FILE_BASENAME(files_in[i]),hwp_value[i],pixel_value[i]
			;
			;	get reference pixel value (this takes care of gravity wave modulations in total power)
			;
			sub_image = image[511-40:511+40,511-40:511+40]
			rr = median_filtered_mean(sub_image)
			reference_pixel_value[i] = 25000.0 < (rr[0] > (-1000.0))		; to trap bad second readouts
			if(reference_pixel_value[i] ne -1000.0 and reference_pixel_value[i] ne 25000.00) then begin
				s_reference_pixel_value[i] = rr[1] / sqrt(float(N_elements(sub_image))-1)
			endif else begin
				s_reference_pixel_value[i] = 10000.0
				bad_image_score[i] += 40
			endelse
			;
			the_ind[i] = i
		endif else begin
			pixel_value[i] = -1000.0
			reference_pixel_value[i] = -1000.0
			;
			;	no bump to bad image score, since already accounted for hwp=-99
			;
		endelse
	endfor
	;
	;	establish dither number from JD, HWP, FILENAMES
	;
	if(check_dither) then begin
		dither_vector = get_dither(file_name_list, jd, hwp_value)
		time = 24.0d0 * 60.0d0 * (jd - jd[0])
	endif else begin
		dither_vector = intarr(n_images)
		dither_vector[*] = 0
		time = jd
	endelse
	;
	;	PLOT Window 1 (UL): sky vs jd ***************************************************************
	;
	device,decompose=0
	color_names = ["black", "white", "red", "orange", "yellow", "green", "blue", "violet", "gray"]
	color_values= [10,11,12,13,14,15,16,17,18,19]
	black = 10 & white = 11 & red = 12 & orange = 13 & yellow = 14 & green = 15 & blue = 16 & violet = 17 & gray = 18
	device, decomposed=0
	loadct, 13, /silent
	tvlct,[  0,255,255,255,255, 50,  0,148,233], $
	      [  0,255,  0,165,200,185,  0,  0,233], $
	      [  0,255,  0,  0,  0, 50,255,211,233], 10
	;
	;window, draw_wid_0, title='Mean Sky Flux vs Time'
	wset, win_no_0
	zz = median_filtered_mean(pixel_value)
	ymin = zz[0]-3.0*zz[1]
	ymax = zz[0]+3.0*zz[1]
	;
	;xr = [image_number[0],image_number[n_images-1]]
	xr = [time[0],time[n_images-1]]
	dxr = xr[1] - xr[0]
	xr = [xr[0] - dxr *0.025, xr[1] + dxr * 0.025]
	;
	plot, time, pixel_value, yrange=[ymin, ymax], psym=10, xtitle='Time [min.]',$
		ytitle='Sky Value', background=black, color=white, xrange = xr, xstyle = 1, /NODATA
	oplot, time, pixel_value, psym=10, color=orange, thick=1
	;
	;	plot mean value as dashed line
	;
	xp = [time[0],time[n_images-1]]
	yp = [zz[0],zz[0]]
	oplot, xp, yp, color=blue, thick=2, linestyle = 2
	;
	;	compute mean reference pixel value
	;
	mean_ref = median_filtered_mean(reference_pixel_value)
	s_mean_ref = mean_ref[1]
	;
	;	flag bad images with vertical red lines
	;
	ind_bd = WHERE(bad_image_score[0:(n_images-1)] gt 0,n_bd)
	if(n_bd gt 0) then begin
		for kk = 0, n_bd-1 do begin
			this_i = ind_bd[kk]
			xbd = [1.,1.]*time[this_i]
			ybd = [ymin,ymax]
			oplot, xbd, ybd, psym=-3,symsize=0.001,thick=3,linestyle=1,color=red
		endfor
	endif
	;
	;	PLOT Window 2 (CL): (coloring by dither)
	;
	;
	;	divide by reference pixel value and rescale to mean reference pixel value
	;
	dif_pix = (pixel_value / reference_pixel_value) * mean_ref[0]
	;
	ind_bad = WHERE(pixel_value le -1000.0 or pixel_value ge 25000.0 or $
					reference_pixel_value le -1000.0 or reference_pixel_value ge 25000.0,nbad)
	if(nbad gt 0) then begin
		dif_pix[ind_bad] = -1000.0
		if(debug eq 1) then print, 'Bad pixel values in images ',ind_bad
		for kk = 0, nbad-1 do begin
			bad_image_score[ind_bad[kk]] += 20	; presumably already found...
		endfor
	endif
	;
	wset, win_no_1
	rr = median_filtered_mean(dif_pix)
	ymin = rr[0]-2.0*rr[1]
	ymax = rr[0]+2.0*rr[1]
	;
	plot, time, dif_pix, yrange=[ymin, ymax], psym=10, xtitle='Time [min]',$
		ytitle='Scaled Pixel Value', background=black, color=white, xrange=xr, $
		xstyle=1, /NODATA
	;
	ind = SORT(hwp_value)
	the_hwp_values = hwp_value[UNIQ(hwp_value,SORT(hwp_value))]
	;
	;	trim out the special case of -99.9 if present
	;
	ind_not_99 = WHERE(the_hwp_values ne -99.9,n_not_99)
	if(n_not_99 gt 0) then begin
		;
		the_hwp_values = the_hwp_values[ind_not_99]
	endif else begin
		print, 'ERROR - got a -99 in HWP values. Stopping'
		stop
	endelse
	;
	;	if not 16 HWP positions, the rest of the code won't work...
	;
	if(N_ELEMENTS(the_hwp_values) ne 16) then begin
		;
		fidelity_out[*] = -1
		special_PA = -99.9
		;
		zprint_text, Event, "ERROR - only "+STRING(N_ELEMENTS(the_hwp_values))+" HWP values present - skipping rest of analysis"
		goto, the_end
		;
	endif
	;
	;	how many dither positions?
	;
	the_dither_set = dither_vector(UNIQ(dither_vector,SORT(dither_vector)))
	n_dithers = N_ELEMENTS(the_dither_set)
	;
	WIDGET_CONTROL, text_wid_1, SET_VALUE=STRING(n_dithers)
	;
;	print, "sorted HWP values = ",the_hwp_values
;	print, "number of dither positions = ",n_dithers
	;
	color_order = [white,blue,red,violet,yellow,green,gray,orange]
	nco = N_ELEMENTS(color_order)
	;
	case (*m_ptr).wave_band of
		'H': begin
				nominal_pa = 62.0
				pivot_center1 = 90.0 +62.0 -26.0	; split between 10, 62 for ZH
				pivot_center2 = pivot_center1 - 180.0
			 end
		'K': begin
				nominal_pa = 132.0
				pivot_center1 = 90.0 + 132.0
				pivot_center2 = pivot_center1 - 180.0
			 end
	endcase
	;
	q = fltarr(4)
	u = fltarr(4)
	p = fltarr(4)
	pa = fltarr(4)
	;
	offset = 0.0					; or mean_ref[0]
	;
	q_dith = dblarr(n_dithers)
	s_q_dith = dblarr(n_dithers)
	u_dith = dblarr(n_dithers)
	s_u_dith = dblarr(n_dithers)
	p_dith = dblarr(n_dithers)
	s_p_dith = dblarr(n_dithers)
	pa_dith = dblarr(n_dithers)
	s_pa_dith = dblarr(n_dithers)
	;
	class_by_dither = intarr(n_dithers)
	;
	;	save PAs for all groups and dithers
	;
	n_save = n_dithers * 4
	pa_save = fltarr(n_save)
	time_save = fltarr(n_save)
	set_save = indgen(n_save)
	;
	override = 0
	if(n_dithers gt 0) then begin
		for idith = 0, n_dithers-1 do begin
			;
			ind_d = WHERE(dither_vector eq the_dither_set[idith] and dif_pix ne -1000.0,n_in_dither)	; indices of original file list
			;
			if(n_in_dither gt 0) then begin
			   ;
			   ;   short dither?
			   ;
			   if(n_in_dither le 8) then begin
			       p_dith[idith] = -99.9
  			     pa_dith[idith] = -99.9
  			     s_p_dith[idith] = 99.9
  			     s_pa_dith[idith] = 99.9
  			     ;
  			 endif else begin
  				;
  				cindex = idith MOD nco
  				;
  				oplot, time[ind_d],dif_pix[ind_d],color=color_order[cindex],psym=10,thick=1
  				;
  				;	compute P, PA values for each Q,U set in each dither
  				;
  				for iset = 0, 3 do begin
  					;
  					q[*] = -99.9
  					u[*] = -99.9
  					p[*] = -99.9
  					pa[*] = -99.9
  					;
  					ioff = 4 * iset
  					;
  					hwp_90 = the_hwp_values[ioff+2]
  					ind_90 = WHERE(hwp_value[ind_d] eq hwp_90,n_90)
  					if(n_90 gt 0) then begin
  						all_ind_90 = ind_d[ind_90]
  						this_fold_angle[all_ind_90] = 90
  					endif
  					case n_90 of
  						0: I_90 = -99.9
  						1: I_90 = dif_pix[all_ind_90] + offset
  						else: begin
  								zz = median_filtered_mean(dif_pix[all_ind_90])
  								I_90 = zz[0] + offset
  							   end
  					endcase
  					;
  					hwp_0 = the_hwp_values[ioff]
  					ind_0 = WHERE(hwp_value[ind_d] eq hwp_0,n_0)
  					if(n_0 gt 0) then begin
  						all_ind_0 = ind_d[ind_0]
  						this_fold_angle[all_ind_0] = 0
  					endif
  					case n_0 of
  						0: I_0 = -99.9
  						1: I_0 = dif_pix[all_ind_0] + offset
  						else: begin
  								zz = median_filtered_mean(dif_pix[all_ind_0])
  								I_0 = zz[0] + offset
  							   end
  					endcase
  					;
  					hwp_135 = the_hwp_values[ioff+3]
  					ind_135 = WHERE(hwp_value[ind_d] eq hwp_135,n_135)
  					if(n_135 gt 0) then begin
  						all_ind_135 = ind_d[ind_135]
  						this_fold_angle[all_ind_135] = 135
  					endif
  					case n_135 of
  						0: I_135 = -99.9
  						1: I_135 = dif_pix[all_ind_135] + offset
  						else: begin
  								zz = median_filtered_mean(dif_pix[all_ind_135])
  								I_135 = zz[0] + offset
  							   end
    					endcase
    					;
    					hwp_45 = the_hwp_values[ioff+1]
    					ind_45 = WHERE(hwp_value[ind_d] eq hwp_45,n_45)
    					if(n_45 gt 0) then begin
    						all_ind_45 = ind_d[ind_45]
    						this_fold_angle[all_ind_45] = 45
    					endif
    					case n_45 of
    						0: I_45 = -99.9
    						1: I_45 = dif_pix[all_ind_45] + offset
    						else: begin
    								zz = median_filtered_mean(dif_pix[all_ind_45])
    								I_45 = zz[0] + offset
    							   end
    					endcase
    					;
    					if(I_90 ne -99.9 and I_0 ne -99.9) then begin
    						q[iset] = (I_90 - I_0) / (I_90 + I_0)
    					endif else begin
    						q[iset] = -99.9
    						if(I_90 eq -99.9 and (n_in_dither -1) ge ioff) then bad_image_score[ind_d[ioff]] += 2	; the 'other' image
    						if(I_0 eq -99.9 and (n_in_dither -1) ge (ioff+2)) then bad_image_score[ind_d[ioff+2]] += 2
    					endelse
    					;
    					if(I_135 ne -99.9 and I_45 ne -99.9) then begin
    						u[iset] = (I_135 - I_45) / (I_135 + I_45)
    					endif else begin
    						u[iset] = -99.9
    						if(I_135 eq -99.9 and (n_in_dither -1) ge (ioff+1)) then bad_image_score[ind_d[ioff+1]] += 2
    						if(I_45 eq -99.9 and (n_in_dither -1) ge (ioff+3)) then bad_image_score[ind_d[ioff+3]] += 2
    					endelse
    					;
    					if(q[iset] ne -99.9 and u[iset] ne -99.9) then begin
    						p[iset] = sqrt(q[iset]^2 + u[iset]^2)
    						pa[iset] = (28.65 * ATAN(u[iset],q[iset]) + 72000.0) MOD 180.0
    						if(pa[iset] gt pivot_center1) then pa[iset] -= 180.0
    						if(pa[iset] lt pivot_center2) then pa[iset] += 180.0
    					endif else begin
    						p[iset] = -99.9
    						pa[iset] = -99.9
    						if(ioff le (n_in_dither-1)) then begin
    						    bad_image_score[ind_d[ioff:((ioff+3)<(n_in_dither - 1))]] += 1	; just identifying all 4 maybe bad
    						endif
    					endelse
    					;
    					;	save results for each constituent image
    					;
    					case n_0 of
    						0: 				; don't do anything, as results already deleted
    						else: begin
    							this_q[all_ind_0] = q[iset]
    							this_u[all_ind_0] = u[iset]
    							this_p[all_ind_0] = p[iset]
    							this_pa[all_ind_0] = pa[iset]
    							this_dither_cindex[all_ind_0] = cindex
    						   end
    					endcase
    					;
    					case n_90 of
    						0: 				; don't do anything, as results already deleted
    						else: begin
    							this_q[all_ind_90] = q[iset]
    							this_u[all_ind_90] = u[iset]
    							this_p[all_ind_90] = p[iset]
    							this_pa[all_ind_90] = pa[iset]
    							this_dither_cindex[all_ind_90] = cindex
    						   end
    					endcase
    					;
    					case n_45 of
    						0: 				; don't do anything, as results already deleted
    						else: begin
    							this_u[all_ind_45] = u[iset]
    							this_q[all_ind_45] = q[iset]
    							this_p[all_ind_45] = p[iset]
    							this_pa[all_ind_45] = pa[iset]
    							this_dither_cindex[all_ind_45] = cindex
    						   end
    					endcase
    					;
    					case n_135 of
    						0: 				; don't do anything, as results already deleted
    						else: begin
    							this_u[all_ind_135] = u[iset]
    							this_q[all_ind_135] = q[iset]
    							this_p[all_ind_135] = p[iset]
    							this_pa[all_ind_135] = pa[iset]
    							this_dither_cindex[all_ind_135] = cindex
    						   end
    					endcase
    					;
    					if(debug eq 1) then print, "for dither = ",idith+1,"  for iset = ",iset+1, "  Pol, PA = ",p[iset],pa[iset]
    					if(debug eq 1) then print, "               Q, U = ",q[iset],u[iset]
  				endfor
  				;
  				;	compute folded p, pa for this dither position
  				;
  				ind_0 = WHERE(this_fold_angle[ind_d] eq 0, n_0)
  				ind_45 = WHERE(this_fold_angle[ind_d] eq 45, n_45)
  				ind_90 = WHERE(this_fold_angle[ind_d] eq 90, n_90)
  				ind_135 = WHERE(this_fold_angle[ind_d] eq 135, n_135)
  				;
  				if(n_0 gt 0 and n_90 gt 0 and n_45 gt 0 and n_135 gt 0) then begin
  					if(n_0 gt 1) then begin
  						zz = MOMENT(dif_pix[ind_d[ind_0]])
  					endif else zz = [dif_pix[ind_d[ind_0]],dif_pix[ind_d[ind_0]]]
  					I_0 = zz[0]
  					s_I_0 = sqrt(zz[1])
  					if(n_0 gt 2) then s_I_0 /= sqrt(n_0 - 1.0)
  					if(n_90 gt 1) then begin
  						zz = MOMENT(dif_pix[ind_d[ind_90]])
  					endif else zz = [dif_pix[ind_d[ind_90]],dif_pix[ind_d[ind_90]]]
  					I_90 = zz[0]
  					s_I_90 = sqrt(zz[1])
  					if(n_90 gt 2) then s_I_90 /= sqrt(n_90 - 1.0)
  					if(n_45 gt 1) then begin
  						zz = MOMENT(dif_pix[ind_d[ind_45]])
  					endif else zz = [dif_pix[ind_d[ind_45]],dif_pix[ind_d[ind_45]]]
  					I_45 = zz[0]
  					s_I_45 = sqrt(zz[1])
  					if(n_45 gt 2) then s_I_45 /= sqrt(n_45 - 1.0)
  					if(n_135 gt 1) then begin
  						zz = MOMENT(dif_pix[ind_d[ind_135]])
  					endif else zz = [dif_pix[ind_d[ind_135]],dif_pix[ind_d[ind_135]]]
  					I_135 = zz[0]
  					s_I_135 = sqrt(zz[1])
  					if(n_135 gt 2) then s_I_135 /= sqrt(n_135 - 1.0)
  					;
  					if(I_90 ne -99.9 and I_0 ne -99.9) then begin
  						q_dith[idith] = (I_90 - I_0) / (I_90 + I_0)
  						s_q_dith[idith] = sqrt(s_I_0^2 + s_I_90^2) / (I_90 + I_0)
  					endif else begin
  						q_dith[idith] = -99.9
  						s_q_dith[idith] = 99.9
  					endelse
  					;
  					if(I_135 ne -99.9 and I_45 ne -99.9) then begin
  						u_dith[idith] = (I_135 - I_45) / (I_135 + I_45)
  						s_u_dith[idith] = sqrt(s_I_45^2 + s_I_135^2) / (I_135 + I_45)
  					endif else begin
  						u_dith[idith] = -99.9
  						s_u_dith[idith] = 99.9
  					endelse
  					;
  					if(q_dith[idith] ne -99.9 and u_dith[idith] ne -99.9) then begin
  						p_dith[idith] = sqrt(q_dith[idith]^2 + u_dith[idith]^2)
  						pa_dith[idith] = 28.65 * ATAN(u_dith[idith],q_dith[idith])
  						pa_dith[idith] = (pa_dith[idith] + 72000.0) MOD 180.0
  						if(pa_dith[idith] gt pivot_center1) then pa_dith[idith] -= 180.0
  						if(pa_dith[idith] lt pivot_center2) then pa_dith[idith] += 180.0
  						;
  						s_p_dith[idith] = (1./p_dith[idith]) * sqrt( (u_dith[idith] * s_u_dith[idith])^2 $
  													+ (q_dith[idith] * s_q_dith[idith])^2)
  						s_pa_dith[idith] = (28.65 * s_p_dith[idith] / p_dith[idith]) < 60.0
  					endif else begin
  						p_dith[idith] = -99.9
  						pa_dith[idith] = -99.9
  						s_p_dith[idith] = 99.9
  						s_pa_dith[idith] = 99.9
  					endelse
  					if(debug eq 1) then print, "For dither ",idith+1," P, PA = ",p_dith[idith],'+/-',s_p_dith[idith],'   ',$
  										pa_dith[idith],'+/-',s_pa_dith[idith]
  					if(debug eq 1) then print, "              Q, U = ",q_dith[idith],"+/-",s_q_dith[idith],"     ",u_dith[idith],"+/-",s_u_dith[idith]
  					if(debug eq 1) then print, 'number of data values n_0, n_90, n_45, n_135 = ',n_0, n_90, n_45, n_135
  					;
  					;	for H band, look for Zero Hop
  					;
  					if( (*m_ptr).wave_band eq 'H') then begin
  						test_0a = (pa_dith[idith] - (nominal_pa + 90.0) + 720.0) MOD 180.	; 62 deg
  						test_0a -= 90.0
  						test_0b = test_0a + 180.0
  						test_0 = abs(test_0a) < abs(test_0b)
  						;
  						test_1a = (pa_dith[idith] - (nominal_pa + 90. +45.) + 720.) MOD 180. ; 10 deg
  						test_1a -= 90.0
  						test_1b = test_1a + 180.0
  						test_1 = abs(test_1a) < abs(test_1b)
  						;
  						;	compute probabilities
  						;
  						p_45 = exp(-0.5*( (test_1 / s_pa_dith[idith])^2))
  						p_0  = exp(-0.5*( (test_0 / s_pa_dith[idith])^2))
  						;
  						; condition for insisting on 45 deg shift is that p_45 / p_0 > 2.0 and s_pa < 45 deg
  						;
  						if( (p_45 / p_0) gt 2.0 and s_pa_dith[idith] lt 45.0) then begin
  							class_by_dither[idith] = 1
  							if(debug eq 1) then print, "... 45 degree phase shift needed"
  						endif else begin
  							class_by_dither[idith] = 0
  							if(debug eq 1) then print, "... no phase shift needed"
  						endelse
  					endif
					endif else begin
						class_by_dither[idith] = 0
					endelse
				endelse
			endif else begin
				p_dith[idith] = -99.9
				pa_dith[idith] = -99.9
				s_p_dith[idith] = 99.9
				s_pa_dith[idith] = 99.9
			endelse
		endfor
		;
	endif
	;
	;	circle symbol
	;
	A = FINDGEN(17) * (!PI*2/16.)
	USERSYM, COS(A), SIN(A)
	;
	;	PLOT Window 3 (LL): PA vs jd  ***************************************************************************************
	;
	wset, win_no_2
	rr = median_filtered_mean(this_pa)
	;
	ymin = 0.0   < (MIN(this_pa) - 10.0)
	ymax = 100.0 > (MAX(this_pa) + 10.0)
	plot, time, this_pa, yrange=[ymin, ymax], psym=10, xtitle='Time [min]',$
		ytitle='PA by dither', background=black, color=white, xrange=xr, $
		xstyle=1, /NODATA
	for ii = 0, n_images-1 do begin
		if(this_dither_cindex[ii] ne 10) then plots, time[ii], this_pa[ii], color=color_order[this_dither_cindex[ii]],psym=4,thick=2
	endfor
	;
	;	and overplot the nominal values
	;
	xo = [time[0],time[n_images-1]]
	case (*m_ptr).wave_band of
		'H':	begin
					yg = [60.0,60.0]
					yr = [10.0,10.0]
					oplot, xo, yg, linestyle=2, color=green, thick=2
					oplot, xo, yr, linestyle=2, color=red, thick=2
				end
		'K':	begin
					yg = [132.0,132.0]
					;yr = [10.0,10.0]
					oplot, xo, yg, linestyle=2, color=green, thick=2
				;	oplot, xo, yr, linestyle=2, color=red, thick=2
				end
	endcase
	;
	;	and put bulls eye around any deviating by more than 5 sigma from mean
	;
	ind_dev = WHERE(abs(this_pa - rr[0])/rr[1] gt 5.0, ndev)
	if(ndev gt 0) then begin
		for idev = 0, ndev-1 do begin
			plots, time[ind_dev[idev]], this_pa[ind_dev[idev]], color=red, psym=8, symsize=3.5, thick=2
			plots, time[ind_dev[idev]], this_pa[ind_dev[idev]], color=red, psym=8, symsize=2.5, thick=1
			;
			;	and save info to bad image array
			;
			ind_i = ind_dev[idev]
			bad_image_score[ind_i] += 50
			;
		endfor
	endif
	;
	if(n_dithers gt 1) then begin
		;
		;	PLOT Window 4 (UR): for each dither, plot folded PA values
		;
		wset, win_no_8
		rr = median_filtered_mean(pa_dith)
		ymin = (rr[0]-((2.0*rr[1]) > 10.0)-50.0) > (-50)
		ymax = (rr[0]+((2.0*rr[1]) > 10.0)+50.0) < 230.0
		plot, the_dither_set, pa_dith, yrange=[ymin, ymax], psym=10, xtitle='Dither Number',$
			ytitle='PA (folded) by dither', background=black, color=white, xrange=[0,n_dithers+1], $
			xstyle=1, /NODATA
		for idith = 0, n_dithers-1 do begin
			cindex = idith MOD nco
			plots, the_dither_set[idith], pa_dith[idith], color=color_order[cindex],psym=4,thick=4
			errplot, the_dither_set[idith], (pa_dith[idith] - s_pa_dith[idith]), (pa_dith[idith] + s_pa_dith[idith]), $
				color=color_order[cindex], width=0.04
		endfor
	;
		xo = [the_dither_set[0],the_dither_set[n_dithers-1]]
		oplot, xo, yg, linestyle=2, color=green, thick=2
		if( (*m_ptr).wave_band eq 'H') then oplot, xo, yr, linestyle=2, color=red, thick=2
		;
		;	and put bulls eye around any deviating by more than 6 sigma from mean and more than 1.5 deg
		;
		ind_dev = WHERE(abs(pa_dith - rr[0])/rr[1] gt 6.0 and override eq 0 and rr[1] gt 1.5, ndev)
		if(ndev gt 0) then begin
			for idev = 0, ndev-1 do begin
				plots, the_dither_set[ind_dev[idev]], pa_dith[ind_dev[idev]], color=red, psym=8, symsize=3.5, thick=2
				plots, the_dither_set[ind_dev[idev]], pa_dith[ind_dev[idev]], color=red, psym=8, symsize=2.5, thick=1
				;
				;	look up images in this dither and add badness to their scores
				;
				k_dith = ind_dev[idev]	; dither number
				ind_k_dith = WHERE(dither_vector eq the_dither_set[k_dith],nk_dith)
				if(nk_dith gt 0) then begin
					bad_image_score[ind_k_dith] += 20
				endif
			endfor
		endif
		;
		;	compute distance of each folded PA from the lower value - if any, then box in red
		;
		ind_switch = WHERE( class_by_dither ,nswitch)
		if(nswitch gt 0) then begin
			for idev = 0, nswitch-1 do begin
				this_color = yellow
				if(class_by_dither[ind_switch[idev]] eq 2) then this_color = red
				plots, the_dither_set[ind_switch[idev]], pa_dith[ind_switch[idev]], color=this_color, psym=6, symsize=5, thick=2
				plots, the_dither_set[ind_switch[idev]], pa_dith[ind_switch[idev]], color=this_color, psym=6, symsize=4, thick=1
				;
			endfor
		endif
		;
		;	PLOT Window 5 (CR):	also, look at percentage polarization by dither
		;
		wset, win_no_9
		rr = median_filtered_mean(100.0*p_dith)
		ymin = (rr[0]-((5.0*rr[1]) > 3.0)) > 0.0
		ymax = (rr[0]+((5.0*rr[1]) > 1.0)+3.0) > (MAX(100.0*p_dith))
		plot, the_dither_set, 100.0*p_dith, yrange=[ymin, ymax], psym=10, xtitle='Dither Number',$
			ytitle='P% (folded) by dither', background=black, color=white, xrange=[0,n_dithers+1], $
			xstyle=1, /NODATA
		for idith = 0, n_dithers-1 do begin
			cindex = idith MOD nco
			plots, the_dither_set[idith], 100.0*p_dith[idith], color=color_order[cindex],psym=4,thick=4
			errplot, the_dither_set[idith], 100.*(p_dith[idith] - s_p_dith[idith]), 100.*(p_dith[idith] + s_p_dith[idith]), $
				color=color_order[cindex], width=0.04
		endfor
		;
		xo = [the_dither_set[0],the_dither_set[n_dithers-1]]
		;
		case (*m_ptr).wave_band of
			'H': begin
					yr = [7.0, 7.0]												; for Raw, LC, or flats
					yg = [3.0, 3.0]											; for corrected
					oplot, xo, yg, linestyle=2, color=green, thick=2
					oplot, xo, yr, linestyle=2, color=red, thick=2
				 end
			'K': begin
					yr = [7.0, 7.0]												; for Raw, LC, or flats
					yg = [2.0, 2.0]											; for corrected
					oplot, xo, yg, linestyle=2, color=green, thick=2
					oplot, xo, yr, linestyle=2, color=red, thick=2
				 end
		endcase
		;
		;	and put bulls eye around any deviating by more than 6 sigma from mean *and* by more than 1/2 %
		;
		dpol = abs(100.0 * p_dith - rr[0])
		test_1 =  dpol gt 0.5
		test_2 =  dpol / rr[1] gt 6.0
		ind_dev = WHERE(test_1 and test_2 and pa_dith ne -99.9, ndev)
		if(ndev gt 0) then begin
			for idev = 0, ndev-1 do begin
				plots, the_dither_set[ind_dev[idev]], 100.0*p_dith[ind_dev[idev]], color=red, psym=8, symsize=3.5, thick=2
				plots, the_dither_set[ind_dev[idev]], 100.0*p_dith[ind_dev[idev]], color=red, psym=8, symsize=2.5, thick=1
				;
				;	look up images in this dither and add badness to their scores
				;
				k_dith = ind_dev[idev]	; dither number
				ind_k_dith = WHERE(dither_vector eq the_dither_set[k_dith],nk_dith)
				if(nk_dith gt 0) then begin
					bad_image_score[ind_k_dith] += 5
				endif
			endfor
		endif
	endif
	;
	;	then HWP
	;
	new_pix = dif_pix + offset
	;
	ind_bad = WHERE(dif_pix eq -1000.0,nbad)
	if(nbad gt 0) then new_pix[ind_bad] = -1000.0
	;
	;	next, bin by fold angle and average to find best U, Q, P, PA values
	;
	the_fold_ind = UNIQ(this_fold_angle,SORT(this_fold_angle))
	the_fold_angles = this_fold_angle[the_fold_ind]
	ind_not_99 = WHERE(the_fold_angles ne -99,n_not_99)
	;
	if(n_not_99 gt 0) then begin
		the_fold_angles = the_fold_angles[ind_not_99]
		;
		n_fold = N_ELEMENTS(the_fold_angles)
		if(debug eq 1) then print,"number of HWP fold angles = ",n_fold
		;
		y_fold = dblarr(n_fold)
		s_y_fold = dblarr(n_fold)
		;
		for i_fold = 0, n_fold-1 do begin
			ind_fold = WHERE(this_fold_angle eq the_fold_angles[i_fold],n_this_fold)
			if(n_this_fold ne 0) then begin
				vec = new_pix[ind_fold]
				ind_good = WHERE(vec ne -1000.0,ngood)
				if(ngood gt 0) then vec = vec[ind_good]
				hh = median_filtered_mean(vec)
				y_fold[i_fold] = hh[0]
				s_y_fold[i_fold] = hh[1] / sqrt(float(ngood)-1)
				;
				if(debug eq 1) then print,"i_fold, y_fold, s_y_fold = ",i_fold,y_fold[i_fold],s_y_fold[i_fold]
				;
			endif else begin
				y_fold[i_fold] = -99.9
				s_y_fold[i_fold] = 99.9
			endelse
		endfor
		;
		ind_good = WHERE(y_fold ne -99.9,ngood)
		if(ngood gt 0) then begin
			yave = TOTAL(y_fold[ind_good]) / float(ngood)
			ymax = max(y_fold[ind_good])
			ymin = min(y_fold[ind_good])
			dy = ymax - ymin
			ymax = ymax + 0.1*dy
			ymin = ymin - 0.1*dy
		endif else begin
			;
			;	this is really bad, just jump out
			;
			fidelity_out[*] = -1
			goto, the_end
		endelse
	endif else begin
		;
		;	no good results, jump out
		;
		print,"whoops! no fold angles found - error"
		fidelity_out[*] = -1
		goto, the_end
	endelse
	;
	;	compute P, PA values of the folded set
	;
	I_90 = y_fold[2]
	I_0 = y_fold[0]
	s_I_90 = s_y_fold[2]
	s_I_0 = s_y_fold[0]
	;
	I_135 = y_fold[3]
	I_45 = y_fold[1]
	s_I_135 = s_y_fold[3]
	s_I_45 = s_y_fold[1]
	;
	if(I_0 ne -99.9 and I_90 ne -99.9) then begin
		q_fold = (I_90 - I_0) / (I_90 + I_0)
		N = I_90 - I_0
		D = I_90 + I_0
		s_q_fold = sqrt(s_I_90 * s_I_90 + s_I_0 * s_I_0) / D
	endif else begin
		q_fold = -99.9
		s_q_fold = 99.9
	endelse
	;
	if(I_45 ne -99.9 and I_135 ne -99.9) then begin
		u_fold = (I_135 - I_45) / (I_135 + I_45)
		N = I_135 - I_45
		D = I_135 + I_45
		s_u_fold = sqrt(s_I_135 * s_I_135 + s_I_45 * s_I_45) / D
	endif else begin
		u_fold = -99.9
		s_u_fold = 99.9
	endelse
	;
	if(q_fold ne -99.9 and u_fold ne -99.9) then begin
		p_fold = sqrt(q_fold^2 + u_fold^2)
		pa_fold = (28.65 * ATAN(u_fold,q_fold) + 72000.0) MOD 180
		s_p_fold = sqrt( (u_fold * s_u_fold)^2 + (q_fold * s_q_fold)^2) / p_fold
		s_pa_fold = 28.65 * s_p_fold / p_fold
	endif else begin
		p_fold = -99.9
		pa_fold = -99.9
		s_p_fold = 99.9
		s_pa_fold = 99.9
	endelse
	;
	if(debug eq 1) then print, "Folded Q, U = ",q_fold,"+/-",s_q_fold,"    ",u_fold,"+/-",s_u_fold
	if(debug eq 1) then print, "Folded  Pol, PA = ",p_fold,"+/-",s_p_fold,'   ',pa_fold,"+/-",s_pa_fold
	;
	special_PA = pa_fold
	;
	;window, 12, title='pixel value vs HWP fold angle'
	wset, win_no_3
	;
	xr = [-10., 145.]
	plot, the_fold_angles, y_fold, xrange=xr, xstyle=1, xtitle='HWP fold angle', yrange=[ymin, ymax], psym=4, $
		background=black, color=white, ytitle='Mean Pixel Value', thick=1, symsize=2
	oplot, the_fold_angles, y_fold, color=blue, linestyle=2, thick=3
	;
	for i=0,n_fold-1 do begin
		if(debug eq 1) then print, "ifold, hwp_fold_angle, mean_pixel_value = ",i,the_fold_angles[i],y_fold[i]
	endfor
	;
	;	print angle, pol onto plot
	;
	xyOutS, 15., ymax - 0.2 * (ymax - ymin), STRING(pa_fold,s_pa_fold, format='("PA=",f6.2,"+/-",f5.2," deg.")'), charsize=1.2, color =white
	xyOutS, 15., ymax - 0.45 * (ymax - ymin), STRING(p_fold*100.0, s_p_fold*100.0, format='("P =",f6.2,"+/-",f5.2," %")'), charsize=1.2, color=white
	;
	;	echo to print window
	;
	zprint_text, Event, STRING(pa_fold,s_pa_fold, format='("PA=",f6.2,"+/-",f5.2," deg.")')
	zprint_text, Event, STRING(p_fold*100.0, s_p_fold*100.0, format='("P =",f6.2,"+/-",f5.2," %")')
	;
	colors_dither = strarr(n_dithers)
	;
	if(n_dithers gt 0) then begin
		group_pass = 0
		for i_dith = 0, n_dithers-1 do begin
			ind_d = WHERE(dither_vector eq the_dither_set[i_dith],n_in_dither)	; indices of original file list
			if(n_in_dither gt 0) then begin
				;
				;
				fidelity_out[ind_d] = class_by_dither[i_dith]
				;
				;	which have good classifications?
				;
				case class_by_dither[i_dith] of
					0: colors_dither[i_dith] = "green"
					1: colors_dither[i_dith] = "yellow"
					else: colors_dither[i_dith] = "red"
				endcase
				;
				;	and then *re* classify this dither position, so all images have same recommendation
				;
;				fidelity_out[ind_d] = this_class
				group_pass += class_by_dither[i_dith]
			endif
		endfor
		;
		ZHT_led_stack, (*m_ptr).draw_wid[7], colors_dither
		case group_pass of
			0: color_group = ["green"]
			1: color_group = ["yellow"]
			2: color_group = ["orange"]
			else: color_group = ["red"]
		endcase
		ZHT_led_stack, (*m_ptr).draw_wid[6], color_group
	endif
	;
	;	save bad image data to file
	;
	ind_bad = WHERE(bad_image_score gt 0,n_bad_image)
	;
	if(n_bad_image gt 0) then begin
		;
		;	how many have badness > 10?
		;
		ibad_10 = WHERE(bad_image_score ge 10,n_10)
		;
		print, 'Number with badness > 10 = ',n_10,', number with badness <= 10 = ',n_bad_image-n_10
		;
		if(n_10 gt 0) then begin
			;
			if( (*m_ptr).outdir eq 'none') then begin
					;
					;	select root directory for dumping problem files
					;
					outdir = dialog_pickfile(title='Select Directory for Saving Bad Image Files',/dir)
					(*m_ptr).outdir = outdir
			endif
			;
			;
			outfile = (*m_ptr).outdir + strtrim((*m_ptr).outfile,2) + '_bad_images.dat'		; from name
			;
			openw, lu_out, outfile, /GET_LUN
			(*m_ptr).lu_out = lu_out
			(*m_ptr).outfile = outfile
			;
			printf, lu_out, '   Image                 HWP     dither   Badness_Score'
			printf, lu_out, '-------------------------------------------------------'
			;
			for kbad = 0, n_10-1 do begin
				;
				lu_out = (*m_ptr).lu_out
				;
				if(kbad eq 0) then begin
					printf,lu_out,'  '
					WIDGET_CONTROL, (*m_ptr).text_wid[2], GET_VALUE=field_name
					WIDGET_CONTROL, (*m_ptr).text_wid[3], GET_VALUE=night_desig
					WIDGET_CONTROL, (*m_ptr).text_wid[4], GET_VALUE=starting_image
					WIDGET_CONTROL, (*m_ptr).text_wid[5], GET_VALUE=ending_image
					printf,lu_out,field_name,night_desig,starting_image,ending_image,$
							format='("Field= ",a," Night= ",a," Images (",a," to ",a,")")'
				endif
				;
				kk = ibad_10[kbad]
				;
				printf,lu_out,FILE_BASENAME(files_in[kk]),hwp_value[kk],dither_vector[kk],$
							bad_image_score[kk],'   is bad'
			endfor
			;
			free_lun, lu_out
			;
			(*m_ptr).nbad = n_10
			zprint_text, Event, STRING(n_10, format='("Bad Image Count = ",I3)')
			;
		endif else begin
			(*m_ptr).nbad = 0
		endelse
	endif else begin
		(*m_ptr).nbad = 0
	endelse
	;
	the_end:
	;
end
